
# q-expansion of newform 2738.2.a.h, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2738
weight = 2
poly_data = [-5, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1, 0], [0, 1], [1, 1], [-2, 0], [-1, -1], [2, -1], [-4, 2], [2, 2], [2, -1], [2, -1], [-3, 3], [0, 0], [-8, 1], [6, 0], [-4, 2], [2, 2], [2, 2], [11, -1], [-1, 3], [-2, 4], [-1, -3], [-10, -1], [8, -4], [6, 0], [10, -2], [-4, 2], [5, -1], [0, 3], [-6, 0], [-14, -2], [8, 0], [6, -6], [-9, -3], [8, -3], [-8, -2], [4, -6], [-2, 0], [16, -2], [13, -5], [-8, 4], [-2, -2], [-2, -6], [-3, 3], [4, -8], [-2, 10], [-8, 4], [5, -3], [14, 0], [8, -10], [-4, -6], [0, -9], [-11, -5], [-8, -2], [8, 2], [-6, -6], [20, -4], [6, -6], [22, 0], [-15, -3], [0, 0], [24, 0], [-6, 0], [4, 3], [-18, 3], [22, 4], [18, -6], [22, -2], [-17, 9], [4, -2], [10, 0], [2, 2], [-16, -4], [-16, -6], [2, -6], [4, -9], [4, -8], [-6, 15], [-4, 6], [-6, 6], [20, -4], [4, 1], [-10, 5], [-16, -4], [16, 3], [18, 3], [-19, 5], [10, 4], [0, -12], [2, 8], [-10, 11], [-20, -2], [-2, 1], [-8, 16], [0, -3], [0, 6], [-21, -3], [-8, -2], [2, -4], [-8, 4], [11, -1], [-8, 10], [11, 11], [0, 6], [-38, 4], [-20, -3], [10, -2], [2, -10], [-17, 1], [16, 4], [-17, 9], [14, -7], [28, -12], [32, 5], [19, -9], [10, -5], [-16, -7], [-18, 12], [-2, -11], [6, 3], [-20, -5], [34, 1], [-20, 9], [-2, 4], [28, -2], [-4, 12], [5, 5], [-25, -1], [-14, 4], [3, 9], [-26, 12], [19, -9], [-8, 4], [20, -12], [37, 1], [-16, -1], [-26, -8], [14, -10], [8, 0], [30, -3], [-28, 14], [7, 15], [-4, -4], [-16, 0], [-26, 4], [26, 5], [18, 12], [25, -11], [-26, 4], [-10, -16], [-30, -6], [22, 6], [-3, -9], [14, -10], [22, -20], [16, -8], [-32, -8], [36, 0], [-3, 3], [8, -15], [0, 12], [12, 6], [-1, 17], [13, -5], [45, -3], [16, -14], [4, -8], [10, -5], [18, 0], [11, 9], [13, 1], [20, -4], [18, -3], [-25, 5], [8, -9], [8, 12], [-4, 8], [12, -12], [48, 0], [-2, 18], [-4, -6], [-8, -20], [30, 0], [-9, 3], [-14, 16], [6, 0], [-46, -4], [2, 6], [16, 10], [4, 10], [40, -2], [2, -16], [-36, 0], [52, -2], [-20, -8], [-59, 1], [-50, 4], [-52, 2], [22, -8], [16, 7], [-39, 3], [9, -15], [-22, -6], [-22, 0], [-16, -9], [24, -9], [33, -15], [2, 18], [-9, 15], [4, 22], [-8, 0], [8, 14], [42, -12], [-10, 11], [9, 9], [45, -9], [13, -21], [3, 9], [-10, -10], [-9, -3], [36, 0], [16, 6], [-38, -12], [10, -20], [-30, -9], [13, -11], [26, -7], [36, -15], [40, -20], [38, -6], [18, 12], [40, -6], [-19, 9], [10, 18], [-7, -1], [-29, 15], [38, -10], [20, 15], [-18, 9], [52, -8], [-47, 7], [-46, 2], [-10, -10], [-44, 18], [1, 25], [-36, -3], [18, 21], [-26, 19], [-58, 8], [44, -9], [-8, -8], [15, 3], [-3, 15], [18, 0], [-32, -2], [14, -13], [-28, -7], [-4, 12], [8, 24], [-8, -14], [18, -6], [-22, -7], [10, -8], [-2, 12], [40, 6], [-22, -16], [-56, -9], [-16, -4], [-12, -18], [35, -9], [22, -23], [-23, 13], [22, -14], [-36, 0], [-14, -5], [-4, -3], [20, -6], [-12, -21], [-3, -3], [-8, -9], [16, 4], [-56, 10], [-24, 24], [42, 6], [-68, 6], [52, -14], [-34, 2], [10, 16], [12, 0], [11, 5], [-14, -20], [-18, -12], [-12, -6], [10, 4], [8, -16], [34, 0], [16, -26], [-16, -6], [36, -12], [-22, 8], [-8, -9], [26, 8], [38, 14], [52, -12], [16, -20], [68, 2], [-18, -24], [-42, -6], [-2, -17], [-26, -14], [-26, 25], [14, 2], [4, 16], [-1, -7], [17, 9], [38, -1], [6, -12], [16, 4], [-14, 19], [16, -15], [52, 4], [-2, 22], [1, -9], [33, -3], [4, 0], [-17, 1], [-22, 2], [-52, -3], [48, 6], [-29, 1], [18, 18], [-14, 6], [11, -13], [5, -1], [-16, 2], [-18, -12], [-57, 15], [-50, 0], [10, 4], [-6, -24], [-58, 0], [-26, 12], [4, 13], [-46, -13], [35, 5], [74, 5], [-14, -2], [-50, 12], [4, 24], [-62, 7], [16, 22], [4, 9], [-7, 15], [62, 8], [-48, 12], [40, 0], [0, 21], [18, 3], [10, -20], [41, -13], [35, -31], [14, 23], [-35, -17], [-17, 25], [20, 14], [-56, 3], [20, -16], [-18, -9], [-13, 17], [-52, 2], [-42, 12], [-56, 10], [18, 18], [48, -18], [-14, -11], [-28, 30], [72, -12], [-52, 14], [44, -3], [-78, 12], [-32, -21], [-22, 17], [46, 10], [53, -1], [21, -9], [8, -22], [16, 10], [8, -6], [-35, 13], [4, 22], [-9, -3], [-22, 12], [35, -1], [14, -10], [62, -4], [-26, -2], [46, -12], [23, -7], [47, -7], [7, -9], [0, 18], [20, 0], [-24, 18], [-18, 15], [-24, 6], [34, -23], [38, -6], [-47, 25], [21, -9], [32, 9], [16, 19], [-10, -24], [-4, 26], [-6, 0], [-4, 14], [28, 10], [54, -12], [36, -6], [-26, -6], [54, -12], [26, 5], [-19, -7], [33, 9], [18, -6], [4, 16], [-14, 0], [-28, -4], [3, 3], [-43, 23], [-47, -15], [4, -8]]
